﻿'title:             simple control panel for ic-7700
'author:            H.NAMVA JH4ADK
'origial issued on: 2013.10.09
'last updated on:   2013.10.17

Public Class Form1
    Private RxBuf(1024) As Byte
    Private RxBufLength As Integer
    Private Mode As String
    Private Frequency As Integer
    Private TxBuf(256) As Byte
    Private TxBufLength As Byte



    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        labelFreq.Text = ""
        labelMode.Text = ""
        Me.Mode = ""
        Me.Frequency = 0.0

        If COMport.IsOpen Then
            MessageBox.Show(COMport.PortName & " is already OPENED", "error", _
                            MessageBoxButtons.OK, MessageBoxIcon.Error)
        Else
            COMport.Open()
        End If


    End Sub

    Private Sub COMport_DataReceived(ByVal sender As System.Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles COMport.DataReceived
        Dim len As Integer
        Dim AddLineIntoTxbRx As New AddLineDelegate(AddressOf AddLine)

        Try
            len = COMport.Read(Me.RxBuf, Me.RxBufLength, Me.RxBuf.Length - Me.RxBufLength)
            If len = 0 Then
                Return
            End If
            Me.RxBufLength += len
            TxbRx.Invoke(AddLineIntoTxbRx)

        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub

    Private Sub Form1_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        COMport.DiscardInBuffer()
        COMport.Close()
    End Sub

    Private Function ByteToHex(ByVal bin As Byte) As String
        Dim hex As String = "0123456789ABCDEF"

        ByteToHex = hex.Substring((bin >> 4), 1) & hex.Substring((bin And &HF), 1)
    End Function

    Delegate Sub AddLineDelegate()

    Private Sub FreqAndMode(ByVal str As String)
        Dim enumMode() As String = {"LSB", "USB", "AM", "CW", "RTTY", "FM", "", "CW-R", "RTTY-R", "", "", "", "PSK", "PSK-R"}
        Dim modeIndex As Integer
        Dim FreqStr As String = ""

        If str.Substring(0, 8) <> "FEFE0074" Then
            'IC7700から発せられたブロードキャストメッセージ以外は無視する
            Exit Sub
        End If
        If str.Substring(8, 2) = "00" Or str.Substring(8, 2) = "03" Then
            '周波数データ
            FreqStr = str.Substring(18, 2) & str.Substring(16, 2) & str.Substring(14, 2) _
                & str.Substring(12, 2) & str.Substring(10, 2)
            Me.Frequency = Integer.Parse(FreqStr)
            labelFreq.Text = Me.Frequency.ToString("#,0")

        ElseIf str.Substring(8, 2) = "01" Or str.Substring(8, 2) = "04" Then
            '運用モード
            modeIndex = Integer.Parse(str.Substring(10, 2))
            If modeIndex < enumMode.Length Then
                Me.Mode = enumMode(modeIndex)
                labelMode.Text = Me.Mode
            End If

        End If
    End Sub

    Private Sub AddLine()
        Dim str As String = ""
        Dim temp As String = ""
        For i As Integer = 0 To RxBufLength - 1
            str &= ByteToHex(RxBuf(i))
        Next
        RxBufLength = 0
        TxbRx.AppendText(str & vbCrLf)
        FreqAndMode(str)

        If Len(TxbRx.Text) > (TxbRx.MaxLength - 1000) Then
            'サイズが大きくなり過ぎたら、データを半分消去する
            temp = Mid(TxbRx.Text, TxbRx.MaxLength / 2, Len(TxbRx.Text))
            TxbRx.Text = temp
        End If
    End Sub

    Private Function HexToByte(ByVal hex2digit As String) As Integer
        Dim hexDigit As String = "0123456789ABCDEF"
        Dim value As Integer
        HexToByte = -1
        value = hexDigit.IndexOf(hex2digit(0))
        If value < 0 Then
            Exit Function
        End If
        HexToByte = value * 16
        value = hexDigit.IndexOf(hex2digit(1))
        If value < 0 Then
            Exit Function
        End If
        HexToByte += value
    End Function

    Private Function HexStrToByteArray(ByVal strHex As String, ByRef arrayByte As Byte(), ByRef lenArray As Integer) As Integer
        Dim byteValue As Integer
        HexStrToByteArray = 0
        lenArray = 0
        While strHex.Length > 0
            byteValue = HexToByte(strHex.Substring(0, 2))
            If byteValue < 0 Then
                HexStrToByteArray = -1
                Exit Function
            End If
            arrayByte(lenArray) = byteValue
            lenArray += 1
            HexStrToByteArray += 1
            strHex = strHex.Substring(2)
        End While
    End Function

    Private Sub BtnSend_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnSend.Click
        Dim status As Integer
        'TextBoxSend.Textの内容をIC7700に送信する
        If TextBoxSend.TextLength Mod 2 <> 0 Then
            MsgBox("送信文字列の長さは2の整数倍であるべきです")
            Exit Sub
        End If
        status = HexStrToByteArray(TextBoxSend.Text, TxBuf, TxBufLength)
        If status > 0 Then
            COMport.Write(TxBuf, 0, TxBufLength)
        Else
            MsgBox("文字列を16進数に変換するときにエラーが発生しました")
            Exit Sub
        End If

    End Sub
End Class
